package com.tcctyn.iot.forestfire.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.constant.RegionConstants;
import com.tcctyn.common.core.web.domain.entity.RegionInfo;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.common.core.utils.DateUtils;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.common.core.utils.DistanceCalculationUtil;
import com.tcctyn.common.core.utils.mqtt.MqttPushClient;
import com.tcctyn.iot.forestfire.BO.FireWarningMQTTBO;
import com.tcctyn.iot.forestfire.BO.FireWarningMisjudgeBO;
import com.tcctyn.iot.forestfire.VO.*;
import com.tcctyn.iot.forestfire.domain.FireWarningInfo;
import com.tcctyn.iot.forestfire.domain.PointInfo;
import com.tcctyn.iot.forestfire.domain.Warehouse;
import com.tcctyn.iot.forestfire.enums.DataSourceEnum;
import com.tcctyn.iot.forestfire.enums.FireWarningDisposalStatusEnum;
import com.tcctyn.iot.forestfire.enums.PointTypeEnum;
import com.tcctyn.iot.forestfire.mapper.FireWarningInfoMapper;
import com.tcctyn.iot.forestfire.mapper.PointInfoMapper;
import com.tcctyn.iot.forestfire.mapper.RegionInfoMapper;
import com.tcctyn.iot.forestfire.mapper.WarehouseMapper;
import com.tcctyn.iot.forestfire.searcher.BaseIdListSearcher;
import com.tcctyn.iot.forestfire.searcher.BaseTimeSearcher;
import com.tcctyn.iot.forestfire.searcher.firewarning.FireWarningAnalyzeSearcher;
import com.tcctyn.iot.forestfire.searcher.firewarning.FireWarningConditionSearcher;
import com.tcctyn.iot.forestfire.searcher.schedules.SchedulesSearcher;
import com.tcctyn.iot.forestfire.service.IFireWarningInfoService;
import com.tcctyn.iot.forestfire.service.ISchedulesService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.*;

/**
 * <p>
 * 火情预警信息表 服务实现类
 * </p>
 *
 * @author 代超
 * @since 2024-10-08
 */
@Service
public class FireWarningInfoServiceImpl extends ServiceImpl<FireWarningInfoMapper, FireWarningInfo> implements IFireWarningInfoService {


    @Resource
    private FireWarningInfoMapper fireWarningInfoMapper;

    @Autowired
    private CommonServiceImpl commonService;

    @Resource
    private RegionInfoMapper regionInfoMapper;

    @Autowired
    private MqttPushClient mqttPushClient;

    @Autowired
    private PointInfoMapper pointInfoMapper;

    @Autowired
    private ISchedulesService iSchedulesService;

    @Autowired
    private WarehouseMapper warehouseMapper;

    @Override
    public Page<FireWarningInfo> list(Integer pageNo, Integer pageSize) {
        return fireWarningInfoMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public FireWarningInfo getById(Long id) {
        FireWarningInfo fireWarningInfo = fireWarningInfoMapper.selectById(id);
        return fireWarningInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Integer create(FireWarningInfo fireWarningInfo) {
        //管理后台新增的、后面APP端上报的，数据来源默认为巡护上报
        fireWarningInfo.setDataSource(DataSourceEnum.PATROL_REPORT.name());
        Long regionId = fireWarningInfo.getRegionId();
        if (Objects.nonNull(regionId)) {
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    fireWarningInfo.setForestId(regionId);
                    break;
                case 2:
                    fireWarningInfo.setForestId(Long.valueOf(ancestorsArr[1]));
                    fireWarningInfo.setFarmId(regionId);
                    break;
                case 3:
                    fireWarningInfo.setForestId(Long.valueOf(ancestorsArr[1]));
                    fireWarningInfo.setFarmId(Long.valueOf(ancestorsArr[2]));
                    fireWarningInfo.setFenceId(regionId);
                    break;
                case 4:
                    fireWarningInfo.setForestId(Long.valueOf(ancestorsArr[1]));
                    fireWarningInfo.setFarmId(Long.valueOf(ancestorsArr[2]));
                    fireWarningInfo.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }
        Integer result = fireWarningInfoMapper.insert(fireWarningInfo);
        //新增火情预警后，需要推送消息
        mqttPushClient.publish(1, true, "fireWarning", fireWarningInfo.getFireTitle());

        return result;
    }

    @Override
    public Integer delete(Long id) {
        return fireWarningInfoMapper.deleteById(id);
    }

    @Override
    public Integer update(FireWarningInfo fireWarningInfo) {
        return fireWarningInfoMapper.updateById(fireWarningInfo);
    }

    @Override
    public List<FireWarningInfo> selectList(FireWarningInfo fireWarningInfo) {
        return fireWarningInfoMapper.selectList(new QueryWrapper<FireWarningInfo>().lambda()
                .like(StringUtils.isNotBlank(fireWarningInfo.getFireTitle()), FireWarningInfo::getFireTitle, fireWarningInfo.getFireTitle())
                .eq(StringUtils.isNotBlank(fireWarningInfo.getFireLevel()), FireWarningInfo::getFireLevel, fireWarningInfo.getFireLevel())
                .eq(StringUtils.isNotBlank(fireWarningInfo.getDisposalStatus()), FireWarningInfo::getDisposalStatus, fireWarningInfo.getDisposalStatus())
                .eq(Objects.nonNull(fireWarningInfo.getRegionId()), FireWarningInfo::getRegionId, fireWarningInfo.getRegionId())
                .orderByDesc(FireWarningInfo::getCreatedTime)
        );
    }

    @Override
    public FireWarningCountByStatusVO countByDisposalStatus(FireWarningConditionSearcher searcher) {
        FireWarningCountByStatusVO vo = fireWarningInfoMapper.countByDisposalStatus(searcher);
        return vo;
    }

    @Override
    public List<FireWarningInfoVO> selectByCondition(FireWarningConditionSearcher searcher) {
        List<FireWarningInfo> fireWarningInfoList = fireWarningInfoMapper.selectList(new QueryWrapper<FireWarningInfo>()
                .eq(StringUtils.isNotBlank(searcher.getDisposalStatus()), "disposal_status", searcher.getDisposalStatus())
                .eq(StringUtils.isNotBlank(searcher.getFireLevel()), "fire_level", searcher.getFireLevel())
                .eq(Objects.nonNull(searcher.getForestId()), "forest_id", searcher.getForestId())
                .eq(Objects.nonNull(searcher.getFarmId()), "farm_id", searcher.getFarmId())
                .between(searcher.getStartTime() != null && searcher.getEndTime() != null, "created_time", searcher.getStartTime(), searcher.getEndTime())
        );
        List<FireWarningInfoVO> voList = BeanUtils.copyList(fireWarningInfoList, FireWarningInfoVO.class);
        //给前端地区名称与处理人姓名
        Map<Long, String> regionIdAndNameMap = commonService.regionMapIdToName(null);
        Map<Long, String> userIdAndNameMap = commonService.userMapIdToName(null);
        voList.forEach(vo -> {
            String forestName = regionIdAndNameMap.get(vo.getForestId()) == null ? "" : regionIdAndNameMap.get(vo.getForestId());
            String farmName = regionIdAndNameMap.get(vo.getFarmId()) == null ? "" : regionIdAndNameMap.get(vo.getFarmId());
            String fenceName = regionIdAndNameMap.get(vo.getFenceId()) == null ? "" : regionIdAndNameMap.get(vo.getFenceId());
            vo.setRegionName(forestName + (StringUtils.isEmpty(farmName) ? "" : "/" + farmName) + (StringUtils.isEmpty(fenceName) ? "" : "/" + fenceName));
            vo.setDisposalByName(userIdAndNameMap.get(vo.getDisposalBy()));
        });
        return voList;
    }

    @Override
    public List<FireWarningCountByLevelVO> countByFireLevel(FireWarningConditionSearcher searcher) {
        List<FireWarningCountByLevelVO> voList = fireWarningInfoMapper.countByFireLevel(searcher);
        return voList;
    }

    @Override
    public List<FireWarningInfoVO> selectLimit(FireWarningConditionSearcher searcher) {
        //将未处理的置顶，然后按照时间倒序排列，选取十条数据
        List<FireWarningInfo> fireWarningInfoList = fireWarningInfoMapper.selectList(new QueryWrapper<FireWarningInfo>()
                .lambda().eq(Objects.nonNull(searcher.getForestId()), FireWarningInfo::getForestId, searcher.getForestId())
                .eq(Objects.nonNull(searcher.getFarmId()), FireWarningInfo::getFarmId, searcher.getFarmId())
                .eq(Objects.nonNull(searcher.getFenceId()), FireWarningInfo::getFenceId, searcher.getFenceId())
                .orderByDesc(FireWarningInfo::getDisposalStatus)
                .orderByDesc(FireWarningInfo::getCreatedTime)
                .last("limit 10"));

        List<FireWarningInfoVO> voList = BeanUtils.copyList(fireWarningInfoList, FireWarningInfoVO.class);
        //给前端地区名称与处理人姓名
        Map<Long, String> regionIdAndNameMap = commonService.regionMapIdToName(null);
        Map<Long, String> userIdAndNameMap = commonService.userMapIdToName(null);
        voList.forEach(vo -> {
            String forestName = regionIdAndNameMap.get(vo.getForestId()) == null ? "" : regionIdAndNameMap.get(vo.getForestId());
            String farmName = regionIdAndNameMap.get(vo.getFarmId()) == null ? "" : regionIdAndNameMap.get(vo.getFarmId());
            String fenceName = regionIdAndNameMap.get(vo.getFenceId()) == null ? "" : regionIdAndNameMap.get(vo.getFenceId());
            vo.setRegionName(forestName + (StringUtils.isEmpty(farmName) ? "" : "/" + farmName) + (StringUtils.isEmpty(fenceName) ? "" : "/" + fenceName));
            vo.setDisposalByName(userIdAndNameMap.get(vo.getDisposalBy()));
        });
        return voList;
    }

    @Override
    public FireWarningStatisticsByDataSourceVO statisticsByDataSource(BaseTimeSearcher searcher) {
        FireWarningStatisticsByDataSourceVO vo = fireWarningInfoMapper.statisticsByDataSource(searcher);
        return vo;
    }

    @Override
    public Integer archive(Long id) {
        FireWarningInfo fireWarningInfo = getById(id);
        //火情预警处理过之后才能归档：已研判、已上报后才能归档
        if (FireWarningDisposalStatusEnum.UNHANDLED.name().equals(fireWarningInfo.getDisposalStatus())) {
            throw new ServiceException("火情预警研判或者上报后才能进行归档");
        }
        fireWarningInfo.setDisposalStatus(FireWarningDisposalStatusEnum.ARCHIVED.name());
        return fireWarningInfoMapper.updateById(fireWarningInfo);
    }

    @Override
    public Integer archiveBatch(BaseIdListSearcher<Long> searcher) {
        List<FireWarningInfo> fireWarningInfoList = fireWarningInfoMapper.selectList(new QueryWrapper<FireWarningInfo>()
                .lambda().in(FireWarningInfo::getId, searcher.getIdList())
                .eq(FireWarningInfo::getDisposalStatus, FireWarningDisposalStatusEnum.UNHANDLED.name()));
        if (CollectionUtils.isNotEmpty(fireWarningInfoList)) {
            throw new ServiceException("尚有未处理的火情，请检查后再归档");
        }
        return fireWarningInfoMapper.archiveBatch(searcher.getIdList());
    }

    @Override
    public List<FireWarningInfo> selectDisposedList(FireWarningInfo fireWarningInfo) {
        List<FireWarningInfo> fireWarningInfoList = fireWarningInfoMapper.selectList(new QueryWrapper<FireWarningInfo>().lambda()
                .ne(FireWarningInfo::getDisposalStatus, FireWarningDisposalStatusEnum.UNHANDLED.name())
                .like(StringUtils.isNotBlank(fireWarningInfo.getFireTitle()), FireWarningInfo::getFireTitle, fireWarningInfo.getFireTitle())
                .eq(Objects.nonNull(fireWarningInfo.getRegionId()), FireWarningInfo::getRegionId, fireWarningInfo.getRegionId())
                .orderByDesc(FireWarningInfo::getCreatedTime)
        );
        return fireWarningInfoList;
    }

    @Override
    public Integer analyze(FireWarningAnalyzeSearcher searcher, Long userId) {
        FireWarningInfo fireWarningInfo = new FireWarningInfo();
        BeanUtils.copyBeanProp(searcher, fireWarningInfo);
        fireWarningInfo.setDisposalStatus(FireWarningDisposalStatusEnum.ANALYZED.name());
        fireWarningInfo.setDisposalBy(userId);
        fireWarningInfo.setDisposalTime(new Date());
        return fireWarningInfoMapper.updateById(fireWarningInfo);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void simulate() {
        FireWarningInfo fireWarningInfo = new FireWarningInfo();
        fireWarningInfo.setDeviceNo("BC1733545");
        fireWarningInfo.setDeviceType("CAMERA");
        fireWarningInfo.setAddress("呼马山林场");
        fireWarningInfo.setLongitude("102.77356");
        fireWarningInfo.setLatitude("25.06268");
        fireWarningInfo.setHeight("100");
        fireWarningInfo.setRegionId(17L);
        fireWarningInfo.setForestId(10L);
        fireWarningInfo.setFarmId(17L);
        fireWarningInfo.setFireTitle("呼马山云台01发现疑似火情");
        fireWarningInfo.setFireLevel("3");
        fireWarningInfo.setFireCause("疑似有人弃置未灭烟头，已产生较大烟雾");
        fireWarningInfo.setDisposalStatus(FireWarningDisposalStatusEnum.UNHANDLED.name());
        fireWarningInfo.setDataSource(DataSourceEnum.VIDEO_MONITOR.name());
        fireWarningInfo.setRemark("模拟呼马山云台发现");
        fireWarningInfo.setCreatedTime(new Date());
        fireWarningInfoMapper.insert(fireWarningInfo);

        FireWarningMQTTBO fireWarningMQTTBO = new FireWarningMQTTBO();
        BeanUtils.copyBeanProp(fireWarningInfo, fireWarningMQTTBO);
        //查询云台信息
        PointInfo pointInfo = pointInfoMapper.selectByDeviceNo(fireWarningInfo.getDeviceNo());
        fireWarningMQTTBO.setPointId(pointInfo.getId());
        fireWarningMQTTBO.setPointNo(pointInfo.getPointNo());
        fireWarningMQTTBO.setPointName(pointInfo.getPointName());
        fireWarningMQTTBO.setPointAddress(pointInfo.getAddress());
        fireWarningMQTTBO.setPointLongitude(pointInfo.getLongitude());
        fireWarningMQTTBO.setPointLatitude(pointInfo.getLatitude());
        fireWarningMQTTBO.setPointHeight(pointInfo.getHeight());
        fireWarningMQTTBO.setPointType(pointInfo.getPointType());
        Map<Long, String> regionMapIdToName = commonService.regionMapIdToName(fireWarningInfo.getRegionId());
        fireWarningMQTTBO.setRegionName(regionMapIdToName.get(fireWarningInfo.getRegionId()));

        // 产生火情预警后，需要给前端推送火情预警信息
        mqttPushClient.publish(1, false, "fireWarning", JSON.toJSONString(fireWarningMQTTBO));

    }

    @Override
    public FireWarningSurroundingVO surroundingSearch(Long id) {
        //火情预警周边搜索，默认搜索3公里半径范围内的 护林员、仓库、水源地
        //先实现，后续接口慢再进行优化
        FireWarningInfo fireWarningInfo = fireWarningInfoMapper.selectById(id);
        //火情所在经纬度
        String longitude = fireWarningInfo.getLongitude();
        String latitude = fireWarningInfo.getLatitude();

        //查询火情所属区域信息
        RegionInfo regionInfo = regionInfoMapper.selectById(fireWarningInfo.getRegionId());
        String belongingUnit = regionInfo.getBelongingUnit();
        String unitTel = regionInfo.getUnitTel();
        List<SurroundingPersonVO> personVOList = new ArrayList<>();
        List<SurroundingWareHouseVO> wareHouseVOList = new ArrayList<>();
        List<SurroundingWaterSourceVO> waterSourceVOList = new ArrayList<>();

        //人员的位置信息，存在当前位置用当前位置，否则用默认位置 ,查询当天的人员位置
        SchedulesSearcher schedulesSearcher = new SchedulesSearcher();
        schedulesSearcher.setStartTime(DateUtils.toDate(LocalDate.now().atStartOfDay()));
        schedulesSearcher.setEndTime(DateUtils.toDate(LocalDate.now().atTime(LocalTime.MAX)));
        List<RangerInfoVO> rangerInfoVOList = iSchedulesService.queryRangerInfo(schedulesSearcher);
        if (!CollectionUtils.isEmpty(rangerInfoVOList)) {
            for (RangerInfoVO rangerInfoVO : rangerInfoVOList) {
                String lon2 = rangerInfoVO.getCurrentLongitude() == null ? rangerInfoVO.getDefaultLongitude() : rangerInfoVO.getCurrentLongitude();
                String lat2 = rangerInfoVO.getCurrentLatitude() == null ? rangerInfoVO.getDefaultLatitude() : rangerInfoVO.getCurrentLatitude();
                double distance = DistanceCalculationUtil.calculateDistance(longitude, latitude, lon2, lat2);
                if (Double.compare(distance, 2.0d) == -1) {
                    SurroundingPersonVO surroundingPersonVO = new SurroundingPersonVO();
                    surroundingPersonVO.setUserId(rangerInfoVO.getUserId());
                    surroundingPersonVO.setUserName(rangerInfoVO.getUserName());
                    surroundingPersonVO.setPhonenumber(rangerInfoVO.getPhonenumber());
                    surroundingPersonVO.setLongitude(lon2);
                    surroundingPersonVO.setLatitude(lat2);
                    personVOList.add(surroundingPersonVO);
                }
            }
        }

        //查询仓库列表，筛选仓库
        List<Warehouse> warehouseList = warehouseMapper.selectList(new QueryWrapper<Warehouse>().lambda());
        if (!CollectionUtils.isEmpty(warehouseList)) {
            for (Warehouse warehouse : warehouseList) {
                if (Double.compare(DistanceCalculationUtil.calculateDistance(longitude, latitude, warehouse.getLongitude(), warehouse.getLatitude()), 2.0d) == -1) {
                    SurroundingWareHouseVO surroundingWareHouseVO = new SurroundingWareHouseVO();
                    surroundingWareHouseVO.setWareHouseId(warehouse.getWarehouseId());
                    surroundingWareHouseVO.setWareHouseName(warehouse.getWarehouseName());
                    surroundingWareHouseVO.setLongitude(warehouse.getLongitude());
                    surroundingWareHouseVO.setLatitude(warehouse.getLatitude());
                    wareHouseVOList.add(surroundingWareHouseVO);
                }
            }
        }

        //查询水源地列表，筛选水源地
        List<PointInfo> waterSourceList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().lambda()
                .eq(PointInfo::getPointType, PointTypeEnum.WATER_SOURCE.name()));
        if (!CollectionUtils.isEmpty(waterSourceList)) {
            for (PointInfo waterSource : waterSourceList) {
                if (Double.compare(DistanceCalculationUtil.calculateDistance(longitude, latitude, waterSource.getLongitude(), waterSource.getLatitude()), 2.0d) == -1) {
                    SurroundingWaterSourceVO surroundingWaterSourceVO = new SurroundingWaterSourceVO();
                    surroundingWaterSourceVO.setWaterSourceId(waterSource.getId());
                    surroundingWaterSourceVO.setWaterSourceName(waterSource.getPointName());
                    surroundingWaterSourceVO.setLongitude(waterSource.getLongitude());
                    surroundingWaterSourceVO.setLatitude(waterSource.getLatitude());
                    waterSourceVOList.add(surroundingWaterSourceVO);
                }
            }
        }
        FireWarningSurroundingVO vo = new FireWarningSurroundingVO();
        vo.setBelongingUnit(belongingUnit);
        vo.setUnitTel(unitTel);
        vo.setPersonVOList(personVOList);
        vo.setWareHouseVOList(wareHouseVOList);
        vo.setWaterSourceVOList(waterSourceVOList);
        return vo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Integer misjudge(Long id) {
        FireWarningInfo fireWarningInfo = fireWarningInfoMapper.selectById(id);
        String disposalStatus = fireWarningInfo.getDisposalStatus();
        if (!FireWarningDisposalStatusEnum.UNHANDLED.name().equals(disposalStatus)) {
            throw new ServiceException("该火情预警信息已经处理，请检查");
        }
        FireWarningInfo updateFireWarningInfo = new FireWarningInfo();
        updateFireWarningInfo.setId(id);
        updateFireWarningInfo.setMisjudgeFlag(1);
        updateFireWarningInfo.setDisposalStatus(FireWarningDisposalStatusEnum.ANALYZED.name());
        Integer result = fireWarningInfoMapper.updateById(updateFireWarningInfo);


        //推送已研判消息
        FireWarningMisjudgeBO misjudgeBO = new FireWarningMisjudgeBO();
        misjudgeBO.setId(id);
        misjudgeBO.setDisposalStatus(FireWarningDisposalStatusEnum.ANALYZED.name());
        misjudgeBO.setMisjudgeFlag(1);
        misjudgeBO.setDisposalMsg("该火情预警信息已确认为误报消息，已作处理");
        mqttPushClient.publish(1, false, "fireWarning", JSON.toJSONString(misjudgeBO));

        return result;
    }

}
