package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.web.domain.AjaxResult;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.MaterialStatisticsVO;
import com.tcctyn.iot.forestfire.domain.Inventory;
import com.tcctyn.iot.forestfire.domain.Material;
import com.tcctyn.iot.forestfire.domain.Warehouse;
import com.tcctyn.iot.forestfire.mapper.InventoryMapper;
import com.tcctyn.iot.forestfire.mapper.MaterialMapper;
import com.tcctyn.iot.forestfire.mapper.WarehouseMapper;
import com.tcctyn.iot.forestfire.searcher.material.MaterialAllocateSearcher;
import com.tcctyn.iot.forestfire.searcher.material.MaterialSearcher;
import com.tcctyn.iot.forestfire.service.IMaterialService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 物资信息 服务实现类
 * </p>
 *
 * @author liuzh
 * @since 2024-10-09
 */
@Service
public class MaterialServiceImpl extends ServiceImpl<MaterialMapper, Material> implements IMaterialService {


    @Resource
    private MaterialMapper materialMapper;

    @Resource
    private InventoryMapper inventoryMapper;

    @Resource
    private WarehouseMapper warehouseMapper;

    @Override
    public List<Material> list(MaterialSearcher materialSearcher) {
        // 如果林场id和仓库id都传了，则查询该仓库的物资
        if (Objects.nonNull(materialSearcher.getRegionId()) && Objects.nonNull(materialSearcher.getWarehouseId())) {
            return materialMapper.selectList(new QueryWrapper<Material>().lambda()
                    .eq(Material::getWarehouseId, materialSearcher.getWarehouseId())
                    .orderByDesc(Material::getCreatedTime));
        }
        // 如果只传了林场id，则查询该林场全部仓库的物资
        if (Objects.isNull(materialSearcher.getWarehouseId()) && Objects.nonNull(materialSearcher.getRegionId())) {
            List<Warehouse> warehouseList = warehouseMapper.selectList(new QueryWrapper<Warehouse>().eq("region_id", materialSearcher.getRegionId()));
            if (warehouseList.isEmpty()) {
                return new ArrayList<Material>();
            }
            // 从 warehouseList 中获取仓库ID
            List<Long> warehouseIdList = warehouseList.stream().map(Warehouse::getWarehouseId).distinct().collect(Collectors.toList());

            return materialMapper.selectList(new QueryWrapper<Material>().lambda()
                    .in(Material::getWarehouseId, warehouseIdList)
                    .orderByDesc(Material::getCreatedTime));
        }
        // 如果没有林场id，但是有仓库id
        if (Objects.nonNull(materialSearcher.getWarehouseId())) {
            return materialMapper.selectList(new QueryWrapper<Material>().lambda()
                    .eq(Material::getWarehouseId, materialSearcher.getWarehouseId())
                    .orderByDesc(Material::getCreatedTime));
        }

        return materialMapper.selectList(new QueryWrapper<Material>().orderByDesc("created_time"));
    }

    @Override
    public Material getById(Long id) {
        Material material = materialMapper.selectById(id);
        return material;
    }

    @Override
    public AjaxResult create(Material material) {
        // 校验物资是否已存在
        Material materialBySerialNumber = materialMapper.selectOne(new QueryWrapper<Material>().lambda()
                .eq(Material::getMaterialName, material.getMaterialName())
                .eq(Material::getMaterialType, material.getMaterialType())
                .eq(Material::getModel, material.getModel())
                .eq(Material::getWarehouseId, material.getWarehouseId())
        );
        if (materialBySerialNumber != null) {
            return AjaxResult.error("物资已存在");
        }

        return materialMapper.insert(material) > 0 ? AjaxResult.success("添加成功") : AjaxResult.error("添加失败");
    }

    @Override
    public Integer delete(Long id) {
        return materialMapper.deleteById(id);
    }

    @Override
    public Integer update(Material material) {
        return materialMapper.updateById(material);
    }

    /**
     * 入库
     *
     * @param materialAllocateSearcher
     * @return
     */
    @Override
    public Integer inbound(MaterialAllocateSearcher materialAllocateSearcher) {
        // 入库时选择好仓库，类型和对应物资
        Material originalMaterial = materialMapper.selectById(materialAllocateSearcher.getMaterialId());
        // 更新对应物资的库存数量
        originalMaterial.setStock(originalMaterial.getStock().add(materialAllocateSearcher.getQuantity()));
        int result = materialMapper.updateById(originalMaterial);
        if (result > 0) {
            // 增加库存记录
            Inventory inventory = new Inventory();
            inventory.setMaterialId(materialAllocateSearcher.getMaterialId());
            inventory.setWarehouseId(originalMaterial.getWarehouseId());
            inventory.setUserId(materialAllocateSearcher.getUserId());
            inventory.setInventoryType(1);
            inventory.setMaterialName(originalMaterial.getMaterialName());
            inventory.setUnit(originalMaterial.getUnit());
            inventory.setQuantity(materialAllocateSearcher.getQuantity());
            inventory.setMaterialType(originalMaterial.getMaterialType());
            inventory.setDescription(materialAllocateSearcher.getDescription());
            inventoryMapper.insert(inventory);
        }

        return result;
    }

    /**
     * 出库
     *
     * @param materialAllocateSearcher
     * @return
     */
    @Override
    public Integer outbound(MaterialAllocateSearcher materialAllocateSearcher) {
        // 领用出库
        Material originalMaterial = materialMapper.selectById(materialAllocateSearcher.getMaterialId());
        if (originalMaterial.getStock().compareTo(materialAllocateSearcher.getQuantity()) < 0) {
            return -1;
        }
        // 更新对应物资的库存数量
        originalMaterial.setStock(originalMaterial.getStock().subtract(materialAllocateSearcher.getQuantity()));
        int result = materialMapper.updateById(originalMaterial);

        if (result > 0) {
            // 增加库存记录
            Inventory inventory = new Inventory();
            inventory.setMaterialId(materialAllocateSearcher.getMaterialId());
            inventory.setWarehouseId(originalMaterial.getWarehouseId());
            inventory.setUserId(materialAllocateSearcher.getUserId());
            inventory.setInventoryType(2);
            inventory.setMaterialName(originalMaterial.getMaterialName());
            inventory.setUnit(originalMaterial.getUnit());
            inventory.setQuantity(materialAllocateSearcher.getQuantity());
            inventory.setMaterialType(originalMaterial.getMaterialType());
            inventory.setDescription(materialAllocateSearcher.getDescription());
            inventoryMapper.insert(inventory);
        }

        return result;
    }

    @Override
    public Integer allocate(MaterialAllocateSearcher materialAllocateSearcher) {
        Material sourceMaterial = materialMapper.selectById(materialAllocateSearcher.getMaterialId());
        if (sourceMaterial.getStock().compareTo(materialAllocateSearcher.getQuantity()) < 0) {
            return -1;
        }
        Material targetMaterial = new Material();
        BeanUtils.copyBeanProp(sourceMaterial, targetMaterial);

        // 如果目标仓库已有该物资，则更新库存数量，否则新增
        Material targetWarehouseMaterial = materialMapper.selectOne(new QueryWrapper<Material>().lambda()
                .eq(Material::getMaterialName, targetMaterial.getMaterialName())
                .eq(Material::getMaterialType, targetMaterial.getMaterialType())
                .eq(Material::getModel, targetMaterial.getModel())
                .eq(Material::getWarehouseId, materialAllocateSearcher.getTargetWarehouseId())
        );
        int result = 0;
        if (targetWarehouseMaterial != null) {
            targetWarehouseMaterial.setStock(targetWarehouseMaterial.getStock().add(materialAllocateSearcher.getQuantity()));
            result += materialMapper.updateById(targetWarehouseMaterial);
        } else {
            targetMaterial.setMaterialId(null);
            targetMaterial.setWarehouseId(materialAllocateSearcher.getTargetWarehouseId());
            targetMaterial.setStock(materialAllocateSearcher.getQuantity());
            result += materialMapper.insert(targetMaterial);
        }
        if (result > 0) {
            // 更新源仓库物资库存数量
            sourceMaterial.setStock(sourceMaterial.getStock().subtract(materialAllocateSearcher.getQuantity()));
            result += materialMapper.updateById(sourceMaterial);
            // 增加库存记录
            Inventory inventoryOut = new Inventory();
            inventoryOut.setWarehouseId(sourceMaterial.getWarehouseId());
            inventoryOut.setMaterialId(sourceMaterial.getMaterialId());
            inventoryOut.setUserId(materialAllocateSearcher.getUserId());
            inventoryOut.setInventoryType(2);
            inventoryOut.setMaterialName(sourceMaterial.getMaterialName());
            inventoryOut.setUnit(sourceMaterial.getUnit());
            inventoryOut.setQuantity(materialAllocateSearcher.getQuantity());
            inventoryOut.setMaterialType(sourceMaterial.getMaterialType());
            inventoryOut.setDescription(materialAllocateSearcher.getDescription());
            Inventory inventoryIn = new Inventory();
            BeanUtils.copyBeanProp(inventoryOut, inventoryIn);
            inventoryIn.setWarehouseId(materialAllocateSearcher.getTargetWarehouseId());
            inventoryIn.setInventoryType(1);
            inventoryMapper.insert(inventoryOut);
            inventoryMapper.insert(inventoryIn);
        }

        return result;
    }

    @Override
    public List<MaterialStatisticsVO> getStatisticsByWarehouse(Long warehouseId) {
        List<MaterialStatisticsVO> voList = new ArrayList<>();
        if (warehouseId != null) {
            QueryWrapper<Material> queryWrapper = new QueryWrapper<>();
            queryWrapper.select("warehouse_id as 'warehouseId'", "material_type as 'materialType'", "count(material_id) as 'materialNum'")
                    .eq("warehouse_id", warehouseId)
                    .groupBy("warehouse_id", "material_type");

            List<Map<String, Object>> list = this.listMaps(queryWrapper);
            voList = BeanUtils.copyList(list, MaterialStatisticsVO.class);
        }

        return voList;
    }

}
