package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.constant.RegionConstants;
import com.tcctyn.common.core.web.domain.TreeSelect;
import com.tcctyn.common.core.web.domain.entity.RegionInfo;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.common.core.utils.SpringUtils;
import com.tcctyn.iot.forestfire.enums.AreaTypeEnum;
import com.tcctyn.iot.forestfire.mapper.RegionInfoMapper;
import com.tcctyn.iot.forestfire.service.IRegionInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 区域信息表 服务实现类
 * </p>
 *
 * @author 代超
 * @since 2024-09-24
 */
@Service
public class RegionInfoServiceImpl extends ServiceImpl<RegionInfoMapper, RegionInfo> implements IRegionInfoService {


    @Resource
    private RegionInfoMapper regionInfoMapper;

    @Override
    public Page<RegionInfo> list(Integer pageNo, Integer pageSize) {
        return regionInfoMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public RegionInfo getById(Long id) {
        RegionInfo regionInfo = regionInfoMapper.selectById(id);
        return regionInfo;
    }

    @Override
    public Integer create(RegionInfo regionInfo) {
        // 需要对地区编号、地区名称做校验
        List<RegionInfo> regionInfoList1 = regionInfoMapper.selectList(new QueryWrapper<RegionInfo>().eq("area_no", regionInfo.getAreaNo()));
        if (!regionInfoList1.isEmpty()) {
            throw new ServiceException(AreaTypeEnum.getByEnumName(regionInfo.getAreaType()).getDesc() + "编号已存在");
        }

        List<RegionInfo> regionInfoList2 = regionInfoMapper.selectList(new QueryWrapper<RegionInfo>().eq("area_name", regionInfo.getAreaName()));
        if (!regionInfoList2.isEmpty()) {
            throw new ServiceException(AreaTypeEnum.getByEnumName(regionInfo.getAreaType()).getDesc() + "名称已存在");
        }
        if (regionInfo.getParentId() == 0) {
            regionInfo.setAncestors("0");
        } else {
            regionInfo.setAncestors(regionInfoMapper.selectById(regionInfo.getParentId()).getAncestors() + RegionConstants.DIVISION_COMMA + regionInfo.getParentId());
        }

        return regionInfoMapper.insert(regionInfo);
    }

    @Override
    public Integer delete(Long id) {
        return regionInfoMapper.deleteById(id);
    }

    @Override
    public Integer update(RegionInfo regionInfo) {
        // 需要对地区编号、地区名称做校验
        List<RegionInfo> regionInfoList1 = regionInfoMapper.selectList(new QueryWrapper<RegionInfo>().eq("area_no", regionInfo.getAreaNo()).ne("id", regionInfo.getId()));
        if (!regionInfoList1.isEmpty()) {
            throw new ServiceException(AreaTypeEnum.getByEnumName(regionInfo.getAreaType()).getDesc() + "编号已存在");
        }

        List<RegionInfo> regionInfoList2 = regionInfoMapper.selectList(new QueryWrapper<RegionInfo>().eq("area_name", regionInfo.getAreaName()).ne("id", regionInfo.getId()));
        if (!regionInfoList2.isEmpty()) {
            throw new ServiceException(AreaTypeEnum.getByEnumName(regionInfo.getAreaType()).getDesc() + "名称已存在");
        }
        if (regionInfo.getParentId() == 0) {
            regionInfo.setAncestors("0");
        } else {
            //如果存在跨级的调整时，以父级的全路径为准
            regionInfo.setAncestors(regionInfoMapper.selectById(regionInfo.getParentId()).getAncestors() + RegionConstants.DIVISION_COMMA + regionInfo.getParentId());
        }
        return regionInfoMapper.updateById(regionInfo);
    }

    @Override
    public List<RegionInfo> selectList(RegionInfo regionInfo) {
        //只对父级id为 0 的林区做分页查询 查询条件：区域名称、区域编号、运营状态、行政区划
        List<RegionInfo> regionInfoList = regionInfoMapper.selectList(new QueryWrapper<RegionInfo>().eq("parent_id", 0)
                .like(StringUtils.isNotBlank(regionInfo.getAreaNo()), "area_no", regionInfo.getAreaNo())
                .like(StringUtils.isNotBlank(regionInfo.getAreaName()), "area_name", regionInfo.getAreaName())
                .like(StringUtils.isNotBlank(regionInfo.getAdministrativeDivision()), "administrative_division", regionInfo.getAdministrativeDivision())
                .eq(StringUtils.isNotBlank(regionInfo.getOperationStatus()), "operation_status", regionInfo.getOperationStatus())
        );
        //判断当前节点下是否有子节点，有子节点给hasChildren置为true
        Map<Long, Boolean> hasChildrenMap = new HashMap<>();
        List<Long> parentIdList = regionInfoList.stream().map(RegionInfo::getId).collect(Collectors.toList());
        if (!parentIdList.isEmpty()) {
            List<RegionInfo> childrenList = regionInfoMapper.selectList(new QueryWrapper<RegionInfo>().lambda().in(RegionInfo::getParentId, parentIdList));
            if (!childrenList.isEmpty()) {
                for (RegionInfo child : childrenList) {
                    hasChildrenMap.put(child.getParentId(), child.isHasChildren());
                }
            }
        }
        for (RegionInfo item : regionInfoList) {
            item.setHasChildren(Objects.isNull(hasChildrenMap.get(item.getId())) ? false : true);
        }
        return regionInfoList;

    }

    @Override
    public List<RegionInfo> selectAllByParentId(Long parentId) {
        List<RegionInfo> regionInfoList = regionInfoMapper.selectList(new QueryWrapper<RegionInfo>().eq("parent_id", parentId));
        Map<Long, Boolean> hasChildrenMap = new HashMap<>();
        List<Long> parentIdList = regionInfoList.stream().map(RegionInfo::getId).collect(Collectors.toList());
        if (!parentIdList.isEmpty()) {
            List<RegionInfo> childrenList = regionInfoMapper.selectList(new QueryWrapper<RegionInfo>().lambda().in(RegionInfo::getParentId, parentIdList));
            if (!childrenList.isEmpty()) {
                for (RegionInfo child : childrenList) {
                    hasChildrenMap.put(child.getParentId(), child.isHasChildren());
                }
            }
        }
        for (RegionInfo item : regionInfoList) {
            item.setHasChildren(Objects.isNull(hasChildrenMap.get(item.getId())) ? false : true);
        }
        return regionInfoList;
    }

    @Override
    public List<RegionInfo> selectAll(Integer level) {
        //实现思路：将所有的数据全部查出来，通过程序来进行递归组装，形成区域信息树
        List<RegionInfo> regionInfoList = regionInfoMapper.selectList(new QueryWrapper<RegionInfo>());
        //从父节点为0开始查询父子级
        List<RegionInfo> rootList = getChildren(regionInfoList, 0L, level);
        return rootList;
    }

    @Override
    public List<RegionInfo> selectAllByAreaType(String areaType) {
        return regionInfoMapper.selectList(new QueryWrapper<RegionInfo>().eq("area_type", areaType));
    }

    @Override
    public List<TreeSelect> selectRegionTree(RegionInfo regionInfo) {
        List<RegionInfo> regionInfoList = SpringUtils.getAopProxy(this).selectRegionList(regionInfo);
        return buildRegionTreeSelect(regionInfoList);
    }

    @Override
    public List<RegionInfo> selectRegionList(RegionInfo regionInfo) {
        return regionInfoMapper.selectList(new QueryWrapper<RegionInfo>().orderByAsc("parent_id"));
    }

    @Override
    public List<TreeSelect> buildRegionTreeSelect(List<RegionInfo> regionInfoList) {
        List<RegionInfo> regionTrees = buildRegionTree(regionInfoList);
        return regionTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    @Override
    public List<RegionInfo> buildRegionTree(List<RegionInfo> regionInfoList) {
        List<RegionInfo> returnList = new ArrayList<RegionInfo>();
        List<Long> tempList = regionInfoList.stream().map(RegionInfo::getId).collect(Collectors.toList());
        for (RegionInfo regionInfo : regionInfoList) {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(regionInfo.getParentId())) {
                recursionFn(regionInfoList, regionInfo);
                returnList.add(regionInfo);
            }
        }
        if (returnList.isEmpty()) {
            returnList = regionInfoList;
        }
        return returnList;
    }

    /**
     * 递归列表
     */
    private void recursionFn(List<RegionInfo> list, RegionInfo t) {
        // 得到子节点列表
        List<RegionInfo> childList = getChildList(list, t);
        t.setChildren(childList);
        for (RegionInfo tChild : childList) {
            if (hasChild(list, tChild)) {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<RegionInfo> getChildList(List<RegionInfo> list, RegionInfo t) {
        List<RegionInfo> tlist = new ArrayList<>();
        Iterator<RegionInfo> it = list.iterator();
        while (it.hasNext()) {
            RegionInfo n = (RegionInfo) it.next();
            if (com.tcctyn.common.core.utils.StringUtils.isNotNull(n.getParentId()) && n.getParentId().longValue() == t.getId().longValue()) {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<RegionInfo> list, RegionInfo t) {
        return getChildList(list, t).size() > 0;
    }


    /**
     * 递归遍历根节点集合，组装子节点
     * 可以选择层级，没有层级时用null值，有层级需要传入相应的层级即可；层级至少为1层
     *
     * @param regionInfoList
     * @param parentId
     * @return
     */
    private List<RegionInfo> getChildren(List<RegionInfo> regionInfoList, Long parentId, Integer level) {
        List<RegionInfo> parentList = regionInfoList.stream().filter(regionInfo1 -> regionInfo1.getParentId().equals(parentId)).collect(Collectors.toList());
        if (level == 0) {
            for (RegionInfo regionInfo : parentList) {
                regionInfo.setChildren(getChildren(regionInfoList, regionInfo.getId(), level));
            }
        } else {
            if (level > 1) {
                for (RegionInfo regionInfo : parentList) {
                    regionInfo.setChildren(getChildren(regionInfoList, regionInfo.getId(), level - 1));
                }
            }
        }
        return parentList;
    }

}
