package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.VO.AirportVO;
import com.tcctyn.iot.forestfire.domain.DeviceInfo;
import com.tcctyn.iot.forestfire.domain.SatelliteDataSimulation;
import com.tcctyn.iot.forestfire.enums.DeviceTypeEnum;
import com.tcctyn.iot.forestfire.mapper.DeviceInfoMapper;
import com.tcctyn.iot.forestfire.mapper.SatelliteDataSimulationMapper;
import com.tcctyn.iot.forestfire.service.ISatelliteDataSimulationService;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 卫星数据模拟表 服务实现类
 * </p>
 *
 * @author 代超
 * @since 2024-11-05
 */
@Service
public class SatelliteDataSimulationServiceImpl extends ServiceImpl<SatelliteDataSimulationMapper, SatelliteDataSimulation> implements ISatelliteDataSimulationService {


    @Resource
    private SatelliteDataSimulationMapper satelliteDataSimulationMapper;

    @Resource
    private DeviceInfoMapper deviceInfoMapper;

    @Override
    public Page<SatelliteDataSimulation> list(Integer pageNo, Integer pageSize) {
        return satelliteDataSimulationMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public SatelliteDataSimulation getById(Long id) {
        SatelliteDataSimulation satelliteDataSimulation = satelliteDataSimulationMapper.selectById(id);
        return satelliteDataSimulation;
    }

    @Override
    public Integer create(SatelliteDataSimulation satelliteDataSimulation) {
        return satelliteDataSimulationMapper.insert(satelliteDataSimulation);
    }

    @Override
    public Integer delete(Long id) {
        return satelliteDataSimulationMapper.deleteById(id);
    }

    @Override
    public Integer update(SatelliteDataSimulation satelliteDataSimulation) {
        return satelliteDataSimulationMapper.updateById(satelliteDataSimulation);
    }

    @Override
    public List<SatelliteDataSimulation> selectRecentlyData(Date currentTime) {
        List<SatelliteDataSimulation> list = satelliteDataSimulationMapper.selectRecentlyData(currentTime);
        return list;
    }

    @Override
    public AirportVO reduceRisk(Long id) {
        SatelliteDataSimulation satelliteDataSimulation = satelliteDataSimulationMapper.selectById(id);
        Double longitude = Double.valueOf(satelliteDataSimulation.getLongitude());
        Double latitude = Double.valueOf(satelliteDataSimulation.getLatitude());
        //查询所有的机场
        List<DeviceInfo> airportList = deviceInfoMapper.selectList(new QueryWrapper<DeviceInfo>().lambda()
                .eq(DeviceInfo::getDeviceType, DeviceTypeEnum.AIRPORT.name())
        );
        List<AirportVO> voList = BeanUtils.copyList(airportList, AirportVO.class);
        AirportVO nearestAirport = null;
        //找出距离最近的机场
        if (!CollectionUtils.isEmpty(voList)) {
            nearestAirport = findNearestAirport(longitude, latitude, voList);
        }
        satelliteDataSimulationMapper.reduceRisk(id);
        return nearestAirport;
    }

    /**
     * 获取两点之间的距离
     * 两点之间的距离，单位为公里
     *
     * @param lon1 第一个点经度
     * @param lat1 第一个点纬度
     * @param lon2 第二个点经度
     * @param lat2 第二个点纬度
     * @return
     */
    public double haversine(double lon1, double lat1, double lon2, double lat2) {
        //地区半径km
        final double EARTH_RADIUS = 6371;
        double dLat = Math.toRadians(lat2 - lat1);
        double dLon = Math.toRadians(lon2 - lon1);

        lat1 = Math.toRadians(lat1);
        lat2 = Math.toRadians(lat2);

        double a = Math.pow(Math.sin(dLat / 2), 2) +
                Math.pow(Math.sin(dLon / 2), 2) * Math.cos(lat1) * Math.cos(lat2);
        double c = 2 * Math.asin(Math.sqrt(a));

        return EARTH_RADIUS * c;
    }

    /**
     * 给出火情经纬度与机场对象List,计算并返回距离最近的机场
     */
    public AirportVO findNearestAirport(Double longitude, Double latitude, List<AirportVO> airportList) {
        AirportVO result = null;
        double minDistance = Double.MAX_VALUE;
        for (AirportVO airport : airportList) {
            double distance = haversine(longitude, latitude, Double.valueOf(airport.getLongitude()), Double.valueOf(airport.getLatitude()));
            if (distance < minDistance) {
                minDistance = distance;
                result = airport;
            }
        }
        return result;
    }

}
