package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yulichang.toolkit.JoinWrappers;
import com.github.yulichang.wrapper.MPJLambdaWrapper;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.common.core.utils.DateUtils;
import com.tcctyn.common.core.utils.StringUtils;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.iot.forestfire.BO.PointInfoBO;
import com.tcctyn.iot.forestfire.BO.PositionBO;
import com.tcctyn.iot.forestfire.BO.SysUserBO;
import com.tcctyn.iot.forestfire.VO.*;
import com.tcctyn.iot.forestfire.domain.*;
import com.tcctyn.iot.forestfire.enums.PointTypeEnum;
import com.tcctyn.iot.forestfire.mapper.*;
import com.tcctyn.iot.forestfire.searcher.schedules.ScheduleSwapSearcher;
import com.tcctyn.iot.forestfire.searcher.schedules.SchedulesSearcher;
import com.tcctyn.iot.forestfire.service.ICommonService;
import com.tcctyn.iot.forestfire.service.IScheduleSwapService;
import com.tcctyn.iot.forestfire.service.ISchedulesService;
import com.tcctyn.system.api.RemoteUserService;
import com.tcctyn.system.api.domain.SysUser;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.YearMonth;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 排班表 服务实现类
 * </p>
 *
 * @author liuzh
 * @since 2024-09-24
 */

@Service
public class SchedulesServiceImpl extends ServiceImpl<SchedulesMapper, Schedules> implements ISchedulesService {

    @Resource
    private SchedulesMapper schedulesMapper;

    @Resource
    private ICommonService commonService;

    @Resource
    private CheckInRecordMapper checkInRecordMapper;

    @Resource
    private PointInfoMapper pointInfoMapper;

    @Resource
    private PatrolPathMapper patrolPathMapper;

    @Resource
    private IScheduleSwapService scheduleSwapService;

    @Resource
    private TemporaryTaskMapper temporaryTaskMapper;

    @Resource
    private RemoteUserService remoteUserService;

    /**
     * 查询一周的排班信息
     *
     * @param schedules
     * @return
     */
    public List<ScheduleDetail> weekList(Schedules schedules) {
        //根据传入日期查一周的排班信息，查询条件增加林场id
        MPJLambdaWrapper<Schedules> mpjLambdaWrapper = JoinWrappers.lambda(Schedules.class)
                .select(Schedules::getScheduleId, Schedules::getUserId, Schedules::getShiftId, Schedules::getPathId, Schedules::getDayOfWeek, Schedules::getDateOfWeek)
                .select(SysUser::getUserName)
                .select(Shift::getShiftName)
                .select(PatrolPath::getPathName)
                .leftJoin(SysUser.class, SysUser::getUserId, Schedules::getUserId)
                .leftJoin(Shift.class, Shift::getShiftId, Schedules::getShiftId)
                .leftJoin(PatrolPath.class, PatrolPath::getId, Schedules::getPathId)
                .between(Schedules::getDateOfWeek, DateUtils.parseDateToStr("yyyy-MM-dd", DateUtils.getThisWeekMonday(schedules.getDateOfWeek())), DateUtils.parseDateToStr("yyyy-MM-dd", DateUtils.getThisWeekSunday(schedules.getDateOfWeek())))
                .eqIfExists(Schedules::getUserId, schedules.getUserId())
                .eqIfExists(PatrolPath::getBelongingRegionId, schedules.getForestId());
        //关联查询用户信息、班次、路线
        List<ScheduleDetail> weeklist = schedulesMapper.selectJoinList(ScheduleDetail.class, mpjLambdaWrapper);
        return weeklist;
    }

    /**
     * 从周排班信息中提取出每个用户的排班信息
     *
     * @param weeklist
     * @return
     */
    @Override
    public List<Map<String, Object>> list(List<ScheduleDetail> weeklist, Schedules schedules) {
        //weeklist按userName分组
//        Map<String,List<SceduleDetail>> groupByUserName = weeklist.stream().collect(Collectors.groupingBy(SceduleDetail::getUserName));
        //weeklist按路线id分组
        Map<Long, List<ScheduleDetail>> groupByPathId = weeklist.stream().collect(Collectors.groupingBy(ScheduleDetail::getPathId));
        List<Map<String, Object>> resultList = new ArrayList<>();
        for (Map.Entry<Long, List<ScheduleDetail>> entry : groupByPathId.entrySet()) {
            TreeMap<String, Object> oneMap = new TreeMap<>();
            oneMap.put("pathId", entry.getKey());
            //获取传入日期所在周的周一
            Date currentMonday = DateUtils.getThisWeekMonday(schedules.getDateOfWeek());
            //从周一开始将这周的日期转为字符串并作为key存入
            oneMap.put(DateUtils.parseDateToStr("yyyy-MM-dd", currentMonday), "");
            for (int i = 1; i <= 6; i++) {
                oneMap.put(DateUtils.parseDateToStr("yyyy-MM-dd", DateUtils.addDays(currentMonday, i)), "");
            }
            //获取分组后的数据
            List<ScheduleDetail> oneList = entry.getValue();
            for (ScheduleDetail s : oneList) {
                oneMap.put("pathName", s.getPathName());
                //用户id拼接
                if (!oneMap.get(s.getDateOfWeek()).toString().isEmpty()) {
                    oneMap.put(s.getDateOfWeek(), oneMap.get(s.getDateOfWeek()) + "," + s.getUserId());
                } else {
                    oneMap.put(s.getDateOfWeek(), String.valueOf(s.getUserId()));
                }
            }
            resultList.add(oneMap);
        }
        return resultList;
    }

    /**
     * 生成下周的排班信息
     *
     * @return
     */
    @Override
    public boolean generate() {
        String nextWeek = DateUtils.parseDateToStr("yyyy-MM-dd", DateUtils.addDays(new Date(), 7));
        //判断下周是否已存在排班信息
        if (schedulesMapper.selectCount(new QueryWrapper<Schedules>().eq("date_of_week", nextWeek)) > 0) {
            return false;
        }
        //查询本周所有排班信息
        List<Schedules> weekList = schedulesMapper.selectList(new QueryWrapper<Schedules>().between("date_of_week", DateUtils.parseDateToStr("yyyy-MM-dd", DateUtils.getThisWeekMonday(new Date())), DateUtils.parseDateToStr("yyyy-MM-dd", DateUtils.getThisWeekSunday(new Date()))));
        //创建一个list保存新的排班信息
        List<Schedules> newList = new ArrayList<>();
        //weekList中每一条记录的日期增加7天
        for (Schedules schedules : weekList) {
            Schedules newSchedules = new Schedules();
            newSchedules.setUserId(schedules.getUserId());
            newSchedules.setShiftId(schedules.getShiftId());
            newSchedules.setPathId(schedules.getPathId());
            newSchedules.setDayOfWeek(schedules.getDayOfWeek());
            newSchedules.setDateOfWeek(DateUtils.addDays(schedules.getDateOfWeek(), 7));
            newList.add(newSchedules);
        }

        return this.saveBatch(newList);

    }

    @Override
    public Schedules getById(Long id) {
        Schedules schedules = schedulesMapper.selectById(id);
        return schedules;
    }

    @Override
    public Integer create(Schedules schedules) {
        return schedulesMapper.insert(schedules);
    }

    @Override
    public Integer delete(Long id) {
        return schedulesMapper.deleteById(id);
    }

    @Override
    public Integer update(Schedules schedules) {
        return schedulesMapper.updateById(schedules);
    }

    /**
     * 批量创建或更新排班信息
     *
     * @param scheduleDetail
     * @return
     */
    @Override
    public Integer createBatch(ScheduleDetail scheduleDetail) {
        int result = 0;
        //传入的userIds拆分，并转换成Long类型的list
        List<Long> newUserIds = Arrays.stream(scheduleDetail.getUserIds().split(",")).map(Long::parseLong).collect(Collectors.toList());
        //根据传入路线id和日期查询用户排班列表
        List<Schedules> schedulesList = schedulesMapper.selectList(new QueryWrapper<Schedules>().eq("path_id", scheduleDetail.getPathId()).eq("date_of_week", scheduleDetail.getDateOfWeek()));

        //如果schedulesList为空，则直接插入
        if (schedulesList.isEmpty()) {
            for (Long userId : newUserIds) {
                Schedules newSchedules = new Schedules();
                newSchedules.setUserId(userId);
                newSchedules.setPathId(scheduleDetail.getPathId());
                newSchedules.setDateOfWeek(DateUtils.parseDate(scheduleDetail.getDateOfWeek()));
                result += schedulesMapper.insert(newSchedules);
            }
            return result;
        } else {
            //从查询结果中获取已排班用户id
            List<Long> oldUserIds = schedulesList.stream().map(Schedules::getUserId).collect(Collectors.toList());
            //用户id取交集: 如果没有交集，则删除旧的，插入新的
            List<Long> intersection = newUserIds.stream().filter(oldUserIds::contains).collect(Collectors.toList());
            if (intersection.isEmpty()) {
                schedulesMapper.delete(new QueryWrapper<Schedules>().eq("path_id", scheduleDetail.getPathId()).eq("date_of_week", scheduleDetail.getDateOfWeek()));
                for (Long userId : newUserIds) {
                    Schedules newSchedules = new Schedules();
                    newSchedules.setUserId(userId);
                    newSchedules.setPathId(scheduleDetail.getPathId());
                    newSchedules.setDateOfWeek(DateUtils.parseDate(scheduleDetail.getDateOfWeek()));
                    result += schedulesMapper.insert(newSchedules);
                }
            } else {
                //有交集的情况，取两个集合中不同的用户id，能查到的删除，不能查到的插入
                List<Long> differenceIds = new ArrayList<>();
                differenceIds.addAll(newUserIds);
                differenceIds.addAll(oldUserIds);
                differenceIds.removeAll(intersection);

                for (Long userId : differenceIds) {
                    if (schedulesMapper.selectCount(new QueryWrapper<Schedules>().eq("user_id", userId).eq("path_id", scheduleDetail.getPathId()).eq("date_of_week", scheduleDetail.getDateOfWeek())) > 0) {
                        schedulesMapper.delete(new QueryWrapper<Schedules>().eq("user_id", userId).eq("path_id", scheduleDetail.getPathId()).eq("date_of_week", scheduleDetail.getDateOfWeek()));
                    } else {
                        Schedules newSchedules = new Schedules();
                        newSchedules.setUserId(userId);
                        newSchedules.setPathId(scheduleDetail.getPathId());
                        newSchedules.setDateOfWeek(DateUtils.parseDate(scheduleDetail.getDateOfWeek()));
                        result += schedulesMapper.insert(newSchedules);
                    }
                }
            }
        }

        return result;
    }

    @Override
    public List<Schedules> selectList(Schedules schedules, Date startDate, Date endDate, String userName) {
        MPJLambdaWrapper<Schedules> queryWrapper = JoinWrappers.lambda(Schedules.class)
                .select(Schedules::getScheduleId, Schedules::getUserId, Schedules::getShiftId, Schedules::getPathId, Schedules::getDayOfWeek, Schedules::getDateOfWeek)
                .leftJoin(SysUser.class, SysUser::getUserId, Schedules::getUserId)
                .between(Objects.nonNull(startDate) && Objects.nonNull(endDate), Schedules::getDateOfWeek, startDate, endDate);

        if (StringUtils.isNotBlank(userName)) {
            queryWrapper.like(SysUser::getUserName, userName);
        }

        return schedulesMapper.selectJoinList(Schedules.class, queryWrapper);
    }



    @Override
    public Integer swapSchedule(ScheduleSwapSearcher scheduleSwapSearcher) {
        //查询申请人的排班信息
        Schedules swapSchedule1 = schedulesMapper.selectOne(new QueryWrapper<Schedules>().eq("user_id", scheduleSwapSearcher.getApplicantId()).eq("date_of_week", scheduleSwapSearcher.getSwapDate()));
        //查询被替换者的排班信息
        Schedules swapSchedule2 = schedulesMapper.selectOne(new QueryWrapper<Schedules>().eq("user_id", scheduleSwapSearcher.getReplacerId()).eq("date_of_week", scheduleSwapSearcher.getBackDate()));

        swapSchedule1.setUserId(scheduleSwapSearcher.getReplacerId());
        swapSchedule1.setSwapFlag(1);
        int result1 = schedulesMapper.updateById(swapSchedule1);
        if (result1 > 0) {
            ScheduleSwap scheduleSwap = new ScheduleSwap();
            scheduleSwap.setScheduleId(swapSchedule1.getScheduleId());
            scheduleSwap.setPathId(swapSchedule1.getPathId());
            scheduleSwap.setBeforeUser(scheduleSwapSearcher.getApplicantId());
            scheduleSwap.setAfterUser(scheduleSwapSearcher.getReplacerId());
            scheduleSwap.setDescription(scheduleSwapSearcher.getReason());
            scheduleSwapService.create(scheduleSwap);
        }

        swapSchedule2.setUserId(scheduleSwapSearcher.getApplicantId());
        swapSchedule2.setSwapFlag(1);
        int result2 = schedulesMapper.updateById(swapSchedule2);
        if (result2 > 0) {
            ScheduleSwap scheduleSwap = new ScheduleSwap();
            scheduleSwap.setScheduleId(swapSchedule2.getScheduleId());
            scheduleSwap.setPathId(swapSchedule2.getPathId());
            scheduleSwap.setBeforeUser(scheduleSwapSearcher.getReplacerId());
            scheduleSwap.setAfterUser(scheduleSwapSearcher.getApplicantId());
            scheduleSwap.setDescription(scheduleSwapSearcher.getReason());
            scheduleSwapService.create(scheduleSwap);
        }

        return result1 + result2;
    }


    @Override
    public List<RangerInfoVO> queryRangerInfo(SchedulesSearcher params) {
        List<Schedules> schedulesList = schedulesMapper.selectList(new QueryWrapper<Schedules>().lambda()
                .eq(Objects.nonNull(params.getForestId()), Schedules::getForestId, params.getForestId())
                .eq(Objects.nonNull(params.getFarmId()), Schedules::getFarmId, params.getFarmId())
                .eq(Objects.nonNull(params.getFenceId()), Schedules::getFenceId, params.getFenceId())
                .between(Objects.nonNull(params.getStartTime()) && Objects.nonNull(params.getEndTime()), Schedules::getDateOfWeek, params.getStartTime(), params.getEndTime()));
        List<Long> pathIdList = schedulesList.stream().map(Schedules::getPathId).distinct().collect(Collectors.toList());

        Map<Long, SysUserBO> userIdAndBOMap = commonService.userMapIdToBO(null);
        //查询当日路径下的打卡记录
        List<CheckInRecord> checkInRecordList = checkInRecordMapper.selectList(new QueryWrapper<CheckInRecord>().lambda()
                .in(!CollectionUtils.isEmpty(pathIdList), CheckInRecord::getPathId, pathIdList)
                .between(Objects.nonNull(params.getStartTime()) && Objects.nonNull(params.getEndTime()), CheckInRecord::getCheckInTime, params.getStartTime(), params.getEndTime())
                .eq(CheckInRecord::getEffectiveFlag, 1)
                .orderByAsc(CheckInRecord::getCheckInTime));
        Map<Long, PositionBO> pathIdAndPositionMap = new HashMap<>(checkInRecordList.size());

        // 上面倒序排，查到的位置信息保留的是最近的一个,组装当前位置信息
        for (CheckInRecord checkInRecord : checkInRecordList) {
            PositionBO positionBO = new PositionBO(checkInRecord.getLongitude(), checkInRecord.getLatitude(), checkInRecord.getHeight());
            pathIdAndPositionMap.put(checkInRecord.getPathId(), positionBO);
        }

        // 巡护路径下的所有点位信息  ，查询巡护点与打卡点
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().lambda()
                .in(!CollectionUtils.isEmpty(pathIdList), PointInfo::getBelongingRegionId, pathIdList).and(p -> {
                    p.eq(PointInfo::getPointType, PointTypeEnum.CHECK_IN_POINT.name()).or().eq(PointInfo::getPointType, PointTypeEnum.PATROL_POINT.name());
                })
        );
        Map<Long, PointInfo> defaultPositionMap = pointInfoList.stream().filter(pointInfo -> Objects.equals(1, pointInfo.getSortNo())).collect(Collectors.toMap(PointInfo::getBelongingRegionId, pointInfo -> pointInfo));

        // 按路径id分组
        Map<Long, List<PointInfo>> pathIdAndPointInfoList = pointInfoList.stream().collect(Collectors.groupingBy(PointInfo::getBelongingRegionId));

        // 查询巡护路径，获取路径名称信息
        List<PatrolPath> pathInfoList = patrolPathMapper.selectList(new QueryWrapper<PatrolPath>().lambda().in(!CollectionUtils.isEmpty(pathIdList), PatrolPath::getId, pathIdList));
        Map<Long, String> pathIdAndNameMap = pathInfoList.stream().collect(Collectors.toMap(PatrolPath::getId, PatrolPath::getPathName));


        //返回数据组装
        List<RangerInfoVO> rangerInfoVOList = new ArrayList<>();
        for (Schedules schedules : schedulesList) {
            RangerInfoVO rangerInfoVO = new RangerInfoVO();
            rangerInfoVO.setUserId(schedules.getUserId());
            rangerInfoVO.setUserName(userIdAndBOMap.get(schedules.getUserId()).getNickName());
            rangerInfoVO.setPhonenumber(userIdAndBOMap.get(schedules.getUserId()).getPhonenumber());

            // 当前位置 ,需要对Map做判空处理
            rangerInfoVO.setCurrentLongitude(pathIdAndPositionMap.get(schedules.getPathId()) == null ? null : pathIdAndPositionMap.get(schedules.getPathId()).getLongitude());
            rangerInfoVO.setCurrentLatitude(pathIdAndPositionMap.get(schedules.getPathId()) == null ? null : pathIdAndPositionMap.get(schedules.getPathId()).getLatitude());
            rangerInfoVO.setCurrentHeight(pathIdAndPositionMap.get(schedules.getPathId()) == null ? null : pathIdAndPositionMap.get(schedules.getPathId()).getHeight());

            // 默认位置
            rangerInfoVO.setDefaultLongitude(defaultPositionMap.get(schedules.getPathId()) == null ? null : defaultPositionMap.get(schedules.getPathId()).getLongitude());
            rangerInfoVO.setDefaultLatitude(defaultPositionMap.get(schedules.getPathId()) == null ? null : defaultPositionMap.get(schedules.getPathId()).getLatitude());
            rangerInfoVO.setDefaultHeight(defaultPositionMap.get(schedules.getPathId()) == null ? null : defaultPositionMap.get(schedules.getPathId()).getHeight());


            //路径id、路径名称
            rangerInfoVO.setPathId(schedules.getPathId());
            rangerInfoVO.setPathName(pathIdAndNameMap.get(schedules.getPathId()));
            rangerInfoVO.setPointInfoList(BeanUtils.copyList(pathIdAndPointInfoList.get(schedules.getPathId()), PointInfoBO.class));

            //路径下的点位信息
            rangerInfoVOList.add(rangerInfoVO);
        }

        return rangerInfoVOList;
    }

    @Override
    public RangerStatisticsVO queryRangerStatistics(SchedulesSearcher params) {
        //查询当天的排班信息，统计人员及巡护路径
        List<Schedules> schedulesList = schedulesMapper.selectList(new QueryWrapper<Schedules>().lambda()
                .eq(Objects.nonNull(params.getForestId()), Schedules::getForestId, params.getForestId())
                .eq(Objects.nonNull(params.getFarmId()), Schedules::getFarmId, params.getFarmId())
                .eq(Objects.nonNull(params.getFenceId()), Schedules::getFenceId, params.getFenceId())
                .between(Objects.nonNull(params.getStartTime()) && Objects.nonNull(params.getEndTime()), Schedules::getDateOfWeek, params.getStartTime(), params.getEndTime())
        );
        List<Long> userIdList = new ArrayList<>();
        List<Long> pathIdList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(schedulesList)) {
            //统计护林员人数 跟 护林路径
            userIdList = schedulesList.stream().map(Schedules::getUserId).distinct().collect(Collectors.toList());
            pathIdList = schedulesList.stream().map(Schedules::getPathId).distinct().collect(Collectors.toList());
        }
        //统计当日排班下的所有路径的巡护点位
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>().lambda()
                .in(!CollectionUtils.isEmpty(pathIdList), PointInfo::getBelongingRegionId, pathIdList)
                .and(p -> {
                    p.eq(PointInfo::getPointType, PointTypeEnum.CHECK_IN_POINT.name()).or().eq(PointInfo::getPointType, PointTypeEnum.PATROL_POINT.name());
                })
        );
        // 统计当日打卡人数
        List<CheckInRecord> checkInRecordList = checkInRecordMapper.selectList(new QueryWrapper<CheckInRecord>().lambda()
                .eq(Objects.nonNull(params.getForestId()), CheckInRecord::getForestId, params.getForestId())
                .eq(Objects.nonNull(params.getFarmId()), CheckInRecord::getFarmId, params.getFarmId())
                .eq(Objects.nonNull(params.getFenceId()), CheckInRecord::getFenceId, params.getFenceId())
                .eq(CheckInRecord::getEffectiveFlag, 1)
                .between(Objects.nonNull(params.getStartTime()) && Objects.nonNull(params.getEndTime()), CheckInRecord::getCheckInTime, params.getStartTime(), params.getEndTime())
        );
        List<Long> checkInUserIdList = new ArrayList<>();
        List<Long> checkInPointIdList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(checkInRecordList)) {
            checkInUserIdList = checkInRecordList.stream().map(CheckInRecord::getUserId).distinct().collect(Collectors.toList());
            checkInPointIdList = checkInRecordList.stream().map(CheckInRecord::getPointId).distinct().collect(Collectors.toList());
        }

        RangerStatisticsVO rangerStatisticsVO = new RangerStatisticsVO();
        rangerStatisticsVO.setSchedulingNum(userIdList.size());
        rangerStatisticsVO.setCheckInNum(checkInUserIdList.size());
        rangerStatisticsVO.setPatrolPointNum(pointInfoList.size());
        rangerStatisticsVO.setCheckInPointNum(checkInPointIdList.size());
        return rangerStatisticsVO;
    }

    @Override
    public List<ScheduleVO> queryScheduleByCondition(SchedulesSearcher params) {
        List<Schedules> schedulesList = schedulesMapper.selectList(new QueryWrapper<Schedules>()
                .lambda().eq(Objects.nonNull(params.getForestId()), Schedules::getForestId, params.getForestId())
                .eq(Objects.nonNull(params.getFarmId()), Schedules::getFarmId, params.getFarmId())
                .eq(Objects.nonNull(params.getFenceId()), Schedules::getFenceId, params.getFenceId())
                .between(Objects.nonNull(params.getStartTime()) && Objects.nonNull(params.getEndTime()), Schedules::getDateOfWeek, params.getStartTime(), params.getEndTime())
        );
        Map<Long, String> pathIdAndNameMap = commonService.pathMapIdToName(null);
        Map<Long, String> userIdAndNameMap = commonService.userMapIdToName(null);
        List<ScheduleVO> voList = BeanUtils.copyList(schedulesList, ScheduleVO.class);
        voList.forEach(scheduleVO -> {
            scheduleVO.setPathName(pathIdAndNameMap.get(scheduleVO.getPathId()));
            scheduleVO.setUserName(userIdAndNameMap.get(scheduleVO.getUserId()));
        });
        return voList;
    }

    @Override
    public ScheduleDetailInfoVO queryScheduleDetail(Long scheduleId) {
        Schedules schedules = schedulesMapper.selectById(scheduleId);
        //查询路径下的所有点位信息
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>()
                .lambda().eq(PointInfo::getBelongingRegionId, schedules.getPathId())
                .and(p -> {
                    p.eq(PointInfo::getPointType, PointTypeEnum.CHECK_IN_POINT.name()).or().eq(PointInfo::getPointType, PointTypeEnum.PATROL_POINT.name());
                })
        );
        //查询路径下的已打卡点
        List<CheckInRecord> checkInRecordList = checkInRecordMapper.selectList(new QueryWrapper<CheckInRecord>()
                .lambda().eq(CheckInRecord::getScheduleId, scheduleId)
                .eq(CheckInRecord::getEffectiveFlag, 1)
        );

        // 获得已打卡的点位
        List<Long> checkedPointList = new ArrayList<>();
        Map<Long, Date> pointIdAndCheckInTimeMap = new HashMap<>(checkInRecordList.size());
        if (!CollectionUtils.isEmpty(checkInRecordList)) {
            checkedPointList = checkInRecordList.stream().map(CheckInRecord::getPointId).distinct().collect(Collectors.toList());
            pointIdAndCheckInTimeMap = checkInRecordList.stream().collect(Collectors.toMap(CheckInRecord::getPointId, CheckInRecord::getCheckInTime));
        }
        List<PointInfoBO> pointInfoBOList = BeanUtils.copyList(pointInfoList, PointInfoBO.class);
        for (PointInfoBO pointInfoBO : pointInfoBOList) {
            if (checkedPointList.contains(pointInfoBO.getId())) {
                pointInfoBO.setCheckInFlag(true);
                pointInfoBO.setCheckInTime(pointIdAndCheckInTimeMap.get(pointInfoBO.getId()));
            }
        }
        ScheduleDetailInfoVO vo = new ScheduleDetailInfoVO();
        BeanUtils.copyProperties(schedules, vo);
        vo.setPointInfoBOList(pointInfoBOList);
        return vo;
    }

    @Override
    public ForesterStatisticsVO queryForesterStatistics(Long regionId) {
        ForesterStatisticsVO foresterStatisticsVO = new ForesterStatisticsVO();

        // 查询当日值班人数
        List<Schedules> schedulesList = schedulesMapper.selectList(new QueryWrapper<Schedules>()
                .lambda().eq(Objects.nonNull(regionId), Schedules::getBelongingRegionId, regionId)
                .eq(Schedules::getDateOfWeek, new Date())
        );
        if (CollectionUtils.isEmpty(schedulesList)) {
            foresterStatisticsVO.setTodayDutyNum(0);
            foresterStatisticsVO.setTodayCheckNum(0);
            foresterStatisticsVO.setTotalCheckNum(0);
            foresterStatisticsVO.setTempTaskNum(0);
            return foresterStatisticsVO;
        }
        // 获取当日值班用户id列表
        List<Long> userIdList = schedulesList.stream().map(Schedules::getUserId).distinct().collect(Collectors.toList());
        foresterStatisticsVO.setTodayDutyNum(userIdList.size());
        // 获取当日值班路线id列表
        List<Long> pathIdList = schedulesList.stream().map(Schedules::getPathId).distinct().collect(Collectors.toList());

        // 查询当日值班用户已打卡数量
        List<CheckInRecord> checkInRecordList = checkInRecordMapper.selectList(new QueryWrapper<CheckInRecord>()
                .lambda().in(CheckInRecord::getUserId, userIdList)
                .eq(CheckInRecord::getEffectiveFlag, 1)
                .apply("date_format(check_in_time,'%Y%m%d')=" + "'" + DateUtils.dateTime() + "'")
        );
        foresterStatisticsVO.setTodayCheckNum(checkInRecordList.size());
        // 查询当日总共需要打卡数量
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>()
                .lambda().in(PointInfo::getBelongingRegionId, pathIdList)
                .eq(PointInfo::getPointType, PointTypeEnum.CHECK_IN_POINT.name())
        );
        foresterStatisticsVO.setTotalCheckNum(pointInfoList.size());
        // 查询当日临时任务数量
        List<TemporaryTask> temporaryTaskList = temporaryTaskMapper.selectList(new QueryWrapper<TemporaryTask>()
                .lambda().in(TemporaryTask::getTaskPersonId, userIdList)
                .apply("date_format(issue_time,'%Y%m%d')=" + "'" + DateUtils.dateTime() + "'")
        );
        foresterStatisticsVO.setTempTaskNum(temporaryTaskList.size());
        return foresterStatisticsVO;
    }

    @Override
    public List<UserScheduleStatusVO> getUserScheduleStatus(Long regionId, Date queryDate) {
        List<SysUser> userList = remoteUserService.selectUserByRegionIdList(regionId).getData();
        List<Long> userIds = userList.stream().map(SysUser::getUserId).collect(Collectors.toList());
        if (queryDate == null) {
            queryDate = new Date();
        }
        List<UserScheduleStatusVO> voList = BeanUtils.copyList(userList, UserScheduleStatusVO.class);
        List<Schedules> schedulesList = schedulesMapper.selectList(new QueryWrapper<Schedules>()
                .lambda().in(Schedules::getUserId, userIds)
                .eq(Schedules::getDateOfWeek, queryDate)
        );
        if (!CollectionUtils.isEmpty(schedulesList)) {
            Map<Long, Schedules> schedulesMap = schedulesList.stream().collect(Collectors.toMap(Schedules::getUserId, Function.identity()));
            for (UserScheduleStatusVO vo : voList) {
                Schedules schedules = schedulesMap.get(vo.getUserId());
                if (schedules != null) {
                    vo.setScheduleId(schedules.getScheduleId());
                    vo.setScheduleStatus(schedules.getStatus());
                } else {
                    vo.setScheduleStatus(0);
                }
            }
        }
        return voList;
    }

    @Override
    public ScheduleVO getUserSchedule(Long userId, Date date) {
        //根据用户id获取当日排班
        //当天时间 yyyy-MM-dd,前端没有传时间的话，默认查询当天的，传了时间查询传的那天的
        String today = DateUtils.getDate();
        if (date != null) {
            today = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, date);
        }
        Schedules schedules = schedulesMapper.selectOne(new QueryWrapper<Schedules>().lambda()
                .eq(Schedules::getUserId, userId)
                .eq(Schedules::getDateOfWeek, today)
        );
        if (Objects.isNull(schedules)) {
            throw new ServiceException("没有查到排班信息！");
        }
        //查询已打卡记录
        List<CheckInRecord> checkInRecordList = checkInRecordMapper.selectList(new QueryWrapper<CheckInRecord>().lambda()
                .eq(CheckInRecord::getUserId, userId)
                .eq(CheckInRecord::getScheduleId, schedules.getScheduleId())
                .eq(CheckInRecord::getEffectiveFlag, 1)
        );
        List<PointInfo> pointInfoList = pointInfoMapper.selectList(new QueryWrapper<PointInfo>()
                .lambda().eq(PointInfo::getBelongingRegionId, schedules.getPathId())
                .and(p -> {
                    p.eq(PointInfo::getPointType, PointTypeEnum.CHECK_IN_POINT.name()).or().eq(PointInfo::getPointType, PointTypeEnum.PATROL_POINT.name());
                })
        );
        // 获得已打卡的点位
        List<Long> checkedPointList = new ArrayList<>();
        Map<Long, Date> pointIdAndCheckInTimeMap = new HashMap<>(checkInRecordList.size());
        if (!CollectionUtils.isEmpty(checkInRecordList)) {
            checkedPointList = checkInRecordList.stream().map(CheckInRecord::getPointId).distinct().collect(Collectors.toList());
            pointIdAndCheckInTimeMap = checkInRecordList.stream().collect(Collectors.toMap(CheckInRecord::getPointId, CheckInRecord::getCheckInTime));
        }
        List<PointInfoBO> pointInfoBOList = BeanUtils.copyList(pointInfoList, PointInfoBO.class);
        for (PointInfoBO pointInfoBO : pointInfoBOList) {
            if (checkedPointList.contains(pointInfoBO.getId())) {
                pointInfoBO.setCheckInFlag(true);
                pointInfoBO.setCheckInTime(pointIdAndCheckInTimeMap.get(pointInfoBO.getId()));
            }
        }
        //DO->VO
        ScheduleVO scheduleVO = new ScheduleVO();
        BeanUtils.copyProperties(schedules, scheduleVO);
        Map<Long, String> longStringMap = commonService.pathMapIdToName(scheduleVO.getPathId());
        Map<Long, String> userIdAndNameMap = commonService.userMapIdToName(scheduleVO.getUserId());
        scheduleVO.setPathName(longStringMap.get(scheduleVO.getPathId()));
        scheduleVO.setUserName(userIdAndNameMap.get(scheduleVO.getUserId()));
        scheduleVO.setPointInfoBOList(pointInfoBOList);
        return scheduleVO;
    }

    @Override
    public List<SchedulesMonthVO> queryMonthSchedule(Long userId, YearMonth month) {
        if (Objects.isNull(month)) {
            month = YearMonth.now();
        }
        List<Schedules> schedulesList = schedulesMapper.selectList(new QueryWrapper<Schedules>().lambda()
                .eq(Schedules::getUserId, userId)
                .between(Schedules::getDateOfWeek, month.atDay(1), month.atEndOfMonth())
        );
        List<SchedulesMonthVO> voList = BeanUtils.copyList(schedulesList, SchedulesMonthVO.class);
        return voList;
    }

}

