package com.tcctyn.iot.forestfire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctyn.common.core.constant.RegionConstants;

import com.tcctyn.common.core.web.domain.entity.RegionInfo;
import com.tcctyn.common.core.exception.ServiceException;
import com.tcctyn.common.core.utils.bean.BeanUtils;
import com.tcctyn.common.core.utils.DateUtils;
import com.tcctyn.iot.forestfire.VO.TemporaryTaskVO;
import com.tcctyn.iot.forestfire.domain.TemporaryTask;
import com.tcctyn.iot.forestfire.enums.TaskStatusEnum;
import com.tcctyn.iot.forestfire.mapper.RegionInfoMapper;
import com.tcctyn.iot.forestfire.mapper.TemporaryTaskMapper;
import com.tcctyn.iot.forestfire.searcher.BaseIdListSearcher;
import com.tcctyn.iot.forestfire.searcher.temporarytask.TemporaryTaskStatusSearcher;
import com.tcctyn.iot.forestfire.service.ITemporaryTaskService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 临时任务表 服务实现类
 * </p>
 *
 * @author 代超
 * @since 2024-11-01
 */
@Service
public class TemporaryTaskServiceImpl extends ServiceImpl<TemporaryTaskMapper, TemporaryTask> implements ITemporaryTaskService {


    @Resource
    private TemporaryTaskMapper temporaryTaskMapper;

    @Resource
    private RegionInfoMapper regionInfoMapper;

    @Override
    public Page<TemporaryTask> list(Integer pageNo, Integer pageSize) {
        return temporaryTaskMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public TemporaryTask getById(Long id) {
        TemporaryTask temporaryTask = temporaryTaskMapper.selectById(id);
        return temporaryTask;
    }

    @Override
    public Integer create(TemporaryTask temporaryTask) {
        //新增任务时冗余forest_id,farm_id,fence_id
        Long regionId = temporaryTask.getRegionId();
        if (Objects.nonNull(regionId)) {
            RegionInfo regionInfo = regionInfoMapper.selectById(regionId);
            String ancestors = regionInfo.getAncestors();
            String[] ancestorsArr = ancestors.split(RegionConstants.DIVISION_COMMA);
            switch (ancestorsArr.length) {
                case 1:
                    temporaryTask.setForestId(regionId);
                    break;
                case 2:
                    temporaryTask.setForestId(Long.valueOf(ancestorsArr[1]));
                    temporaryTask.setFarmId(regionId);
                    break;
                case 3:
                    temporaryTask.setForestId(Long.valueOf(ancestorsArr[1]));
                    temporaryTask.setFarmId(Long.valueOf(ancestorsArr[2]));
                    temporaryTask.setFenceId(regionId);
                    break;
                case 4:
                    temporaryTask.setForestId(Long.valueOf(ancestorsArr[1]));
                    temporaryTask.setFarmId(Long.valueOf(ancestorsArr[2]));
                    temporaryTask.setFenceId(Long.valueOf(ancestorsArr[3]));
                    break;
            }
        }
        temporaryTask.setIssueTime(new Date());
        return temporaryTaskMapper.insert(temporaryTask);
    }

    @Override
    public Integer delete(Long id) {
        TemporaryTask temporaryTask = getById(id);
        //只有未受理的任务才可以删除
        if (!TaskStatusEnum.NOT_ACCEPTED.name().equals(temporaryTask.getTaskStatus())) {
            throw new ServiceException("任务已被受理，请勿删除！");
        }
        return temporaryTaskMapper.deleteById(id);
    }

    @Override
    public Integer update(TemporaryTask temporaryTask) {
        return temporaryTaskMapper.updateById(temporaryTask);
    }

    @Override
    public Integer archive(Long id) {
        TemporaryTask temporaryTask = getById(id);
        //任务完成后才可以进行归档
        if (!TaskStatusEnum.COMPLETED.name().equals(temporaryTask.getTaskStatus())) {
            throw new ServiceException("任务完成后才能进行归档");
        }
        temporaryTask.setArchiveStatus(1);
        temporaryTask.setArchiveTime(new Date());
        return temporaryTaskMapper.updateById(temporaryTask);
    }

    @Override
    public Integer archiveBatch(BaseIdListSearcher<Long> searcher) {
        List<TemporaryTask> taskList = temporaryTaskMapper.selectList(new QueryWrapper<TemporaryTask>().lambda()
                .ne(TemporaryTask::getTaskStatus, TaskStatusEnum.COMPLETED.name())
                .in(TemporaryTask::getId, searcher.getIdList())
        );
        if (!CollectionUtils.isEmpty(taskList)) {
            throw new ServiceException("存在未完成任务，请检查");
        }
        return temporaryTaskMapper.archiveBatch(searcher.getIdList());
    }

    @Override
    public List<TemporaryTask> selectList(TemporaryTask temporaryTask) {
        //筛选条件：任务名称、任务类型、任务状态、任务紧急度，按照创建时间进行倒序排序
        List<TemporaryTask> taskList = temporaryTaskMapper.selectList(new QueryWrapper<TemporaryTask>().lambda()
                .like(StringUtils.isNotBlank(temporaryTask.getTaskName()), TemporaryTask::getTaskName, temporaryTask.getTaskName())
                .eq(StringUtils.isNotBlank(temporaryTask.getTaskType()), TemporaryTask::getTaskType, temporaryTask.getTaskType())
                .eq(StringUtils.isNotBlank(temporaryTask.getTaskStatus()), TemporaryTask::getTaskStatus, temporaryTask.getTaskStatus())
                .eq(StringUtils.isNotBlank(temporaryTask.getTaskUrgency()), TemporaryTask::getTaskUrgency, temporaryTask.getTaskUrgency())
                .eq(Objects.nonNull(temporaryTask.getTaskPersonId()), TemporaryTask::getTaskPersonId, temporaryTask.getTaskPersonId())
                .eq(Objects.nonNull(temporaryTask.getIssuerId()), TemporaryTask::getIssuerId, temporaryTask.getIssuerId())
                .eq(Objects.nonNull(temporaryTask.getTaskRecipientId()), TemporaryTask::getTaskRecipientId, temporaryTask.getTaskRecipientId())
                .eq(Objects.nonNull(temporaryTask.getRegionId()), TemporaryTask::getRegionId, temporaryTask.getRegionId())
                .orderByDesc(TemporaryTask::getCreatedTime)
        );
        return taskList;
    }

    @Override
    public List<TemporaryTaskVO> selectOneDayList(TemporaryTask temporaryTask) {
        if (Objects.isNull(temporaryTask.getIssueTime())) {
            temporaryTask.setIssueTime(new Date());
        }
        List<TemporaryTask> taskList = temporaryTaskMapper.selectList(new QueryWrapper<TemporaryTask>().lambda()
                .eq(Objects.nonNull(temporaryTask.getRegionId()), TemporaryTask::getRegionId, temporaryTask.getRegionId())
                .apply("date_format(issue_time,'%Y%m%d')=" + "'" + DateUtils.parseDateToStr("yyyyMMdd", temporaryTask.getIssueTime()) + "'")
                .orderByDesc(TemporaryTask::getCreatedTime));

        List<TemporaryTaskVO> voList = BeanUtils.copyList(taskList, TemporaryTaskVO.class);
        return voList;
    }

    @Override
    public List<TemporaryTask> selectUncompletedList(TemporaryTask temporaryTask) {
        List<TemporaryTask> taskList = temporaryTaskMapper.selectList(new QueryWrapper<TemporaryTask>().lambda()
                .like(StringUtils.isNotBlank(temporaryTask.getTaskName()), TemporaryTask::getTaskName, temporaryTask.getTaskName())
                .eq(Objects.nonNull(temporaryTask.getRegionId()), TemporaryTask::getRegionId, temporaryTask.getRegionId())
                .ne(TemporaryTask::getTaskStatus, TaskStatusEnum.COMPLETED.name())
                .orderByDesc(TemporaryTask::getCreatedTime)
        );
        return taskList;
    }

    @Override
    public Integer updateTaskStatus(TemporaryTaskStatusSearcher searcher, Long userId) {
        TemporaryTask temporaryTask = getById(searcher.getId());
        if (Objects.nonNull(temporaryTask)) {
            if (TaskStatusEnum.NOT_ACCEPTED.name().equals(searcher.getTaskStatus()) && TaskStatusEnum.NOT_ACCEPTED.name().equals(temporaryTask.getTaskStatus())) {
                TemporaryTask acceptTask = new TemporaryTask();
                acceptTask.setId(searcher.getId());
                acceptTask.setTaskStatus(TaskStatusEnum.IN_PROGRESS.name());
                acceptTask.setTaskRecipientId(userId);
                acceptTask.setAcceptTime(new Date());
                return temporaryTaskMapper.updateById(acceptTask);
            } else if (TaskStatusEnum.IN_PROGRESS.name().equals(searcher.getTaskStatus()) && TaskStatusEnum.IN_PROGRESS.name().equals(temporaryTask.getTaskStatus())) {
                TemporaryTask completeTask = new TemporaryTask();
                completeTask.setId(searcher.getId());
                completeTask.setTaskStatus(TaskStatusEnum.COMPLETED.name());
                completeTask.setCompleteTime(new Date());
                return temporaryTaskMapper.updateById(completeTask);
            } else {
                throw new ServiceException("任务状态异常，请检查");
            }
        }
        return 0;
    }

}
